# frozen_string_literal: true

module Bulkrax
  # rubocop:disable Metrics/ModuleLength
  module DatatablesBehavior
    extend ActiveSupport::Concern

    def table_per_page
      per_page = params[:length].to_i
      per_page < 1 ? 30 : per_page
    end

    def order_value(column)
      params['columns']&.[](column)&.[]('data')
    end

    def table_order
      "#{order_value(params&.[]('order')&.[]('0')&.[]('column'))} #{params&.[]('order')&.[]('0')&.[]('dir')}" if params&.[]('order')&.[]('0')&.[]('column').present?
    end

    # convert offset to page number
    def table_page
      params[:start].blank? ? 1 : (params[:start].to_i / params[:length].to_i) + 1
    end

    def entry_table_search
      return @entry_table_search if @entry_table_search
      return @entry_table_search = false if params['search']&.[]('value').blank?

      table_search_value = params['search']&.[]('value')&.downcase

      ['identifier', 'id', 'status_message', 'type', 'updated_at'].map do |col|
        column = Bulkrax::Entry.arel_table[col]
        column = Arel::Nodes::NamedFunction.new('CAST', [column.as('text')])
        column = Arel::Nodes::NamedFunction.new('LOWER', [column])
        @entry_table_search = if @entry_table_search
                                @entry_table_search.or(column.matches("%#{table_search_value}%"))
                              else
                                column.matches("%#{table_search_value}%")
                              end
      end

      @entry_table_search
    end

    def importer_table_search
      return @importer_table_search if @importer_table_search
      return @importer_table_search = false if params['search']&.[]('value').blank?

      table_search_value = params['search']&.[]('value')&.downcase

      ['name', 'id', 'status_message', 'last_error_at', 'last_succeeded_at', 'updated_at'].map do |col|
        column = Bulkrax::Importer.arel_table[col]
        column = Arel::Nodes::NamedFunction.new('CAST', [column.as('text')])
        column = Arel::Nodes::NamedFunction.new('LOWER', [column])
        @importer_table_search = if @importer_table_search
                                   @importer_table_search.or(column.matches("%#{table_search_value}%"))
                                 else
                                   column.matches("%#{table_search_value}%")
                                 end
      end

      @importer_table_search
    end

    def exporter_table_search
      return @exporter_table_search if @exporter_table_search
      return @exporter_table_search = false if params['search']&.[]('value').blank?

      table_search_value = params['search']&.[]('value')&.downcase

      ['name', 'status_message', 'created_at'].map do |col|
        column = Bulkrax::Exporter.arel_table[col]
        column = Arel::Nodes::NamedFunction.new('CAST', [column.as('text')])
        column = Arel::Nodes::NamedFunction.new('LOWER', [column])
        @exporter_table_search = if @exporter_table_search
                                   @exporter_table_search.or(column.matches("%#{table_search_value}%"))
                                 else
                                   column.matches("%#{table_search_value}%")
                                 end
      end

      @exporter_table_search
    end

    def format_importers(importers)
      result = importers.map do |i|
        {
          name: view_context.link_to(i.name, view_context.importer_path(i)),
          status_message: status_message_for(i),
          last_imported_at: i.last_imported_at&.strftime("%b %d, %Y"),
          next_import_at: i.next_import_at&.strftime("%b %d, %Y"),
          enqueued_records: i.last_run&.enqueued_records,
          processed_records: i.last_run&.processed_records || 0,
          failed_records: i.last_run&.failed_records || 0,
          deleted_records: i.last_run&.deleted_records,
          total_collection_entries: i.last_run&.total_collection_entries,
          total_work_entries: i.last_run&.total_work_entries,
          total_complex_subject_entrie: i.last_run&.total_complex_subject_entries,
          total_complex_session_entries: i.last_run&.total_complex_session_entries,
          total_complex_modality_entries: i.last_run&.total_complex_modality_entries,
          total_file_set_entries: i.last_run&.total_file_set_entries,
          actions: importer_util_links(i),
          addtinal_actions: additional_actions(i)
        }
      end
      {
        data: result,
        recordsTotal: Bulkrax::Importer.count,
        recordsFiltered: Bulkrax::Importer.count
      }
    end

    def format_exporters(exporters)
      result = exporters.map do |e|
        {
          name: view_context.link_to(e.name, view_context.exporter_path(e)),
          status_message: status_message_for(e),
          created_at: e.created_at,
          download: download_zip(e),
          actions: exporter_util_links(e)
        }
      end
      {
        data: result,
        recordsTotal: Bulkrax::Exporter.count,
        recordsFiltered: Bulkrax::Exporter.count
      }
    end

    def format_entries(entries, item)
      result = entries.map do |e|
        {
          identifier: view_context.link_to(e.identifier, view_context.item_entry_path(item, e)),
          id: e.id,
          status_message: status_message_for(e),
          type: e.type,
          updated_at: e.updated_at,
          errors: e.status_message == 'Failed' ? view_context.link_to(e.error_class, view_context.item_entry_path(item, e)) : "",
          actions: entry_util_links(e, item)
        }
      end
      {
        data: result,
        recordsTotal: item.entries.size,
        recordsFiltered: item.entries.size
      }
    end

    def entry_util_links(e, item)
      links = []
      links << view_context.link_to(view_context.raw('<span class="fa fa-info-circle"></span>'), view_context.item_entry_path(item, e), class: "mr-2")

      if e.type.eql?('Bulkrax::CrcFolderEntry')

        links << "<a class='fa fa-repeat mr-2' data-toggle='modal' data-target='#bulkraxItemModal' data-entry-id='#{e.id}'></a>" if view_context.an_importer?(item)
        
        verification_status, date, file_path = latest_varification_report_details(e)
        if verification_status.present?
          if verification_status == "Success"
            links << "<span class='fa fa-check-circle text-success mr-2' title='Verified successfully on (#{date})'></span>"
          else
            links << "<span class='fa fa-times-circle text-danger mr-2' title='Verification failed on (#{date})'></span>"
          end
          links << view_context.link_to(view_context.raw('<span class="fa fa-download"></span>'), view_context.url_for(controller: params[:controller], action: "verify_and_generate_work", importer_id: e.importerexporter_id, entry_id: e.id, format: :csv))
        end
    
        if !verification_status || verification_status == "Failure"
          links << view_context.link_to(view_context.raw('<span class="fa fa-refresh"></span>'), view_context.url_for(controller: params[:controller], action: "verify_and_generate_work", importer_id: e.importerexporter_id, entry_id: e.id, verify: true))
        end
      end
      view_context.raw("<div class='d-flex justify-content-between'>#{links.join(" ")}</div>")
    end
    

    def status_message_for(e)
      if e.status_message == "Complete"
        "<td><span class='fa fa-check' style='color: green;'></span> #{e.status_message}</td>"
      elsif e.status_message == "Pending"
        "<td><span class='fa fa-ellipsis-h' style='color: blue;'></span> #{e.status_message}</td>"
      elsif e.status_message == "Skipped"
        "<td><span class='fa fa-step-forward' style='color: yellow;'></span> #{e.status_message}</td>"
      else
        "<td><span class='fa fa-remove' style='color: #{e.status == 'Deleted' ? 'green' : 'red'};'></span> #{e.status_message}</td>"
      end
    end

    def importer_util_links(i)
      links = []
      links << view_context.link_to(view_context.raw('<span class="fa fa-info-circle"></span>'), importer_path(i))
      links << view_context.link_to(view_context.raw('<span class="fa fa-pencil"></span>'), edit_importer_path(i))
      links << view_context.link_to(view_context.raw('<span class="fa fa-remove"></span>'), i, method: :delete, data: { confirm: 'Are you sure?' })
      links.join(" ")
    end

    def exporter_util_links(i)
      links = []
      links << view_context.link_to(view_context.raw('<span class="fa fa-info-circle"></span>'), exporter_path(i))
      links << view_context.link_to(view_context.raw('<span class="fa fa-pencil"></span>'), edit_exporter_path(i), data: { turbolinks: false })
      links << view_context.link_to(view_context.raw('<span class="fa fa-remove"></span>'), i, method: :delete, data: { confirm: 'Are you sure?' })
      links.join(" ")
    end

    def download_zip(e)
      return unless File.exist?(e.exporter_export_zip_path)

      options_html = e.exporter_export_zip_files.flatten.map do |file_name|
        "<option value='#{CGI.escapeHTML(file_name)}'>#{CGI.escapeHTML(file_name)}</option>"
      end.join

      form_html = "<form class='simple_form edit_exporter' id='edit_exporter_#{e.id}' action='#{view_context.exporter_download_path(e)}' accept-charset='UTF-8' method='get'>"
      form_html += "<input name='utf8' type='hidden' value='✓'>"
      form_html += "<select class='btn btn-default form-control' style='width: 200px' name='exporter[exporter_export_zip_files]' id='exporter_#{e.id}_exporter_export_zip_files'>"
      form_html += options_html
      form_html += "</select>\n" # add newline here to add a space between the dropdown and the download button
      form_html += "<input type='submit' name='commit' value='Download' class='btn btn-default'>"
      form_html += "</form>"

      form_html
    end

    def additional_actions(importer)
      # Dropdown menu
      actions_dropdown = view_context.raw("
        <div class='dropdown'>
          <button class='btn btn-secondary dropdown-toggle' type='button' data-toggle='dropdown'>
            More Actions
          </button>
          <ul class='dropdown-menu'>
            <li>#{view_context.link_to("Re-run failed jobs", rerun_failed_entries_path(importer_id: importer.id), method: :post, class: "dropdown-item")}</li>
            <li>#{view_context.link_to("Re-run pending jobs", rerun_pending_entries_path(importer_id: importer.id), method: :post, class: "dropdown-item", data: { confirm: 'Are you sure to re-run pending jobs? It may create the same resource twice.'})}</li>
          </ul>
        </div>
      ")
      actions_dropdown
    end
  end
  # rubocop:enable Metrics/ModuleLength
end