# frozen_string_literal: true
module HyraxHelper
  include ::BlacklightHelper
  include Hyrax::BlacklightOverride
  include Hyrax::HyraxHelperBehavior

  def form_tabs_for(form:)
    if form.instance_of? Hyrax::Forms::BatchUploadForm
      %w[files metadata relationships]
    else
      form.tabs
    end
  end

  def is_tombstonable?(presenter)
    presenter.tombstone_status != ['initiated'] && (is_published?(presenter) || is_archived?(presenter))
  end

  def is_published?(presenter)
    ['deposited', 'published'].include?(presenter.workflow.state)
  end

  def is_archived?(presenter)
    presenter.workflow.state == 'archived'
  end

  def tombstone_out_of_date?(presenter)
    ((Date.today) - presenter.tombstone_date.first.to_date).to_i >= ENV['NUMBER_OF_DAYS_TO_TOMBSTONE'].to_i
  end

  def is_tombston_initiated?(presenter)
    presenter.tombstone_status.present? && presenter.tombstone_status == ['initiated']
  end

  def is_permanently_tombstoned?(presenter)
    presenter.tombstone_status == ['deleted'] && presenter.is_tombstoned
  end

  def can_manage_managers_for_group?(presenter)
    current_user.admin? || current_user.crc_manager_for?(presenter.id)
  end

  def can_manage_members_for_group?(presenter)
    current_user.admin? || current_user.crc_manager_for?(presenter.id) || current_user.group_manager_for?(presenter.id)
  end

  def manager_role(presenter)
    Hyrax::PcdmCollection.find(presenter.id).manager_role
  end

  def member_role(presenter)
    Hyrax::PcdmCollection.find(presenter.id).member_role
  end

  def manager_role_for_group(presenter)
   Hyrax::PcdmCollection.find(presenter.id).manager_role_for_group
  end

  def member_role_for_group(presenter)
    Hyrax::PcdmCollection.find(presenter.id).member_role_for_group
  end

  def can_create_work_in_group?(group_id)
    current_user.admin? || current_user.crc_manager_for?(group_id) || current_user.group_manager_for?(group_id) || current_user.group_member_for?(group_id)
  end

  def truncate_title(title)
    title.truncate(30)
  end

  def show_rerun_action?(importer)
    padding_imports(importer).zero? &&
    importer.last_run &&
    ((importer.last_run.failed_records.positive?) ||
    (importer.parser.total.positive? && importer.last_run.processed_records.zero? ))
  end

  def show_marked_as_complete_action?(importer)
    padding_imports(importer).zero? && 
    importer.last_run && 
    importer.last_run.failed_records.zero? && 
    importer.last_run.enqueued_records.positive?
  end

  def padding_imports(importer)
    importer.entries.left_outer_joins(:latest_status).where('bulkrax_statuses.status_message IS NULL').count
  end

  # This method is used to get a Font Awesome class name or icon image path for different file types.
  # For example:
  #   file_icon_for('document.pdf') #=> "fa fa-file-pdf-o" OR "/pdf_icon.svg"
  def file_icon_for(file)
    extension = File.extname(file).downcase

    case extension
    when ".pdf"
      "fa fa-file-pdf-o"
    when ".doc", ".docx"
      "fa fa-file-word-o"
    when ".xls", ".xlsx", ".csv"
      "fa fa-file-excel-o"
    when ".ppt", ".pptx"
      "fa fa-file-powerpoint-o"
    when ".png", ".jpg", ".jpeg", ".gif", ".svg"
      "fa fa-file-image-o"
    when ".zip", ".rar", ".tar", ".gz"
      "fa fa-file-archive-o"
    when ".mp4", ".avi", ".mov", ".mkv", ".webm", ".wmv"
      "fa fa-file-video-o"
    when ".mp3", ".wav", ".ogg"
      "fa fa-file-audio-o"
    when ".txt", ".md", ".rtf", ".odt"
      "fa fa-file-text-o"
    when ".html", ".htm", ".xml"
      "fa fa-file-code-o"
    else
      "fa fa-file-o"
    end
  end

  def latest_varification_report_details(entry)
   
    # Define the file pattern and search path
    download_path = ENV.fetch('DOWNLOAD_PATH', '/shared/downloads')
    report_pattern = "analysis_report_#{entry.importerexporter_id}_#{entry.id}_*.csv"
    matching_files = Dir.glob(File.join(download_path, report_pattern))
    # Return nil if no matching files are found
    return [nil, nil, nil] if matching_files.empty?
  
    # Find the newest file based on modification time
    newest_file = matching_files.max_by { |file| File.mtime(file) }
  
    # Extract file details
    base_name = File.basename(newest_file, ".*")
    extension = File.extname(newest_file)
    date_time_str, status = base_name.split('_')[-2..]
  
    # Parse the date and time
    date, raw_time = date_time_str.split('T')
    formatted_time = raw_time.gsub('-', ':')
    parsed_datetime = Time.parse("#{date} #{formatted_time}")
  
    [status.titleize, parsed_datetime, newest_file]
  end

  def restricted_file_names?(file_name)
    parsed_file =  YAML.load_file(Rails.root.join('config', 'restricted_files.yml'))
    parsed_file['restricted_file_names'].map(&:downcase).include?(file_name.downcase)
  end

  def s3_file_download_url_for_file_set(file_set_id)
    s3 = S3StorageService.new
    s3.init_client
    file_set = Hyrax::FileSet.find(file_set_id)
    bucket_name = s3.sanitise_name(file_set.parent_work.id)

    s3.presigned_url_for_download(bucket_name, file_set.object_key_on_s3)
  end

  def download_all_button_available?(work_id)
    work = Hyrax.query_service.find_by(id: work_id)
    s3 = S3StorageService.new
    s3.init_client

    meta_object = s3.system_metadata_object(work_id)
    meta_object.present? and (work.date_modified.to_i <= DateTime.parse(meta_object.last_modified.to_s).to_i)
  end
end
