# frozen_string_literal: true

require 'csv'
module Bulkrax
  class CrcFolderParser < ApplicationParser # rubocop:disable Metrics/ClassLength
    include ErroredEntries
    include ExportBehavior
    attr_writer :collections, :file_sets, :works

    def self.export_supported?
      true
    end

    def records(_opts = {})
      return @records if @records.present?

      file_for_import = only_updates ? parser_fields['partial_import_file_path'] : import_file_path

      csv_data = entry_class.read_data(file_for_import)
      importer.parser_fields['total'] = csv_data.count
      importer.save
      
      @records = csv_data.map { |record_data| entry_class.data_for_entry(record_data, nil, self) }
    end

    def build_records
      @collections = []
      @works = []
      @file_sets = []
      @complex_subjects = []
      @complex_sessions = []
      @complex_modalities = []
      if model_field_mappings.map { |mfm| mfm.to_sym.in?(records.first.keys) }.any?
        records.map do |r|
          model_field_mappings.map(&:to_sym).each do |model_mapping|
            next unless r.key?(model_mapping)

            model = r[model_mapping].nil? ? "" : r[model_mapping].strip
            # TODO: Eventually this should be refactored to us Hyrax.config.collection_model
            #       We aren't right now because so many Bulkrax users are in between Fedora and Valkyrie
            if model.casecmp('hyrax::pcdmcollection').zero? || model.casecmp('collectionresource').zero?
              @collections << r
            elsif model.casecmp('hyrax::fileset').zero?
              @file_sets << r
            elsif model.casecmp('crc1280experiment').zero?
              @works << r
            elsif model.casecmp('complexsubject').zero?
              @complex_subjects << r
            elsif model.casecmp('complexsession').zero?
              @complex_sessions << r
            elsif model.casecmp('complexmodality').zero?
              @complex_modalities << r
            end
          end
        end
        @collections = @collections.flatten.compact.uniq
        @file_sets = @file_sets.flatten.compact.uniq
        @works = @works.flatten.compact.uniq
        @complex_subjects = @complex_subjects.flatten.compact.uniq
        @complex_sessions = @complex_sessions.flatten.compact.uniq
        @complex_modalities = @complex_modalities.flatten.compact.uniq
      else # if no model is specified, assume all records are works
        @works = records.flatten.compact.uniq
      end

      true
    end

    def collections
      build_records if @collections.nil?
      @collections
    end

    def works
      build_records if @works.nil?
      @works
    end

    def file_sets
      build_records if @file_sets.nil?
      @file_sets
    end

    def complex_subjects
      build_records if @complex_subjects.nil?
      @complex_subjects
    end

    def complex_sessions
      build_records if @complex_sessions.nil?
      @complex_sessions
    end

    def complex_modalities
      build_records if @complex_modalities.nil?
      @complex_modalities
    end

    def collections_total
      collections.size
    end

    def works_total
      works.size
    end

    def file_sets_total
      file_sets.size
    end

    def complex_subjects_total
      complex_subjects.size
    end

    def complex_sessions_total
      complex_sessions.size
    end

    def complex_modalities_total
      complex_modalities.size
    end

    # We could use CsvEntry#fields_from_data(data) but that would mean re-reading the data
    def import_fields
      @import_fields = [:model,
      :collection_type,
      :crc_collection_type,
      :file,
      :source,
      :parents,
      :for_complex_identifier,
      :for_complex_type,
      :title,
      :contributor,
      :contributor_orcid,
      :contributor_affiliation,
      :contributor_scheme,
      :creator, :creator_orcid,
      :creator_affiliation,
      :creator_scheme,
      :keyword,
      :resource_type,
      :crc_resource_type,
      :folder_name,
      :experiment_description,
      :crc_work_type,
      :doi,
      :alternative_title,
      :description,
      :group_identifier,
      :record_date,
      :date_description,
      :date,
      :modality,
      :subject_identifier,
      :subject_species,
      :subject_type,
      :subject_sex,
      :subject_age,
      :approval_number,
      :subject,
      :publisher,
      :language,
      :funder_identifier,
      :funder_name,
      :award_number,
      :award_uri,
      :award_title,
      :coverage,
      :software_version,
      :extra_information]
    end

    def required_elements?(record)
      missing_elements(record).blank?
    end

    def missing_elements(record)
      keys_from_record = keys_without_numbers(record.reject { |_, v| v.blank? }.keys.compact.uniq.map(&:to_s))
      keys = []
      # Because we're persisting the mapping in the database, these are likely string keys.
      # However, there's no guarantee.  So, we need to ensure that by running stringify.
      importerexporter.mapping.stringify_keys.map do |k, v|
        Array.wrap(v['from']).each do |vf|
          keys << k if keys_from_record.include?(vf)
        end
      end
      required_elements.map(&:to_s) - keys.uniq.map(&:to_s)
    end

    def valid_import?
      compressed_record = records.flat_map(&:to_a).partition { |_, v| !v }.flatten(1).to_h
      error_alert = "Missing at least one required element, missing element(s) are: #{missing_elements(compressed_record).join(', ')}"
      raise StandardError, error_alert unless required_elements?(compressed_record)

      file_paths.is_a?(Array)
    rescue StandardError => e
      set_status_info(e)
      false
    end

    def create_complex_subjects
      create_objects(['complex_subject'])
    end

    def create_complex_sessions
      create_objects(['complex_session'])
    end

    def create_complex_modalities
      create_objects(['complex_modality'])
    end

    def create_objects(types_array = nil)
      index = 0
      file_set_index = 1
      (types_array || %w[collection work complex_subject complex_session complex_modality file_set relationship]).each do |type|
        if type.eql?('relationship')
          ScheduleRelationshipsJob.set(wait: 5.minutes).perform_later(importer_id: importerexporter.id)
          next
        end
        
        send(type.pluralize).each do |current_record|
          next unless record_has_source_identifier(current_record, index)
          break if limit_reached?(limit, index)
          seen[current_record[source_identifier]] = true
          create_entry_and_job(current_record, type, nil, file_set_index)
          increment_counters(index, "#{type}": true)
          index += 1
          file_set_index += 1 if type == 'file_set'
        end
        importer.record_status
      end
      true
    rescue StandardError => e
      set_status_info(e)
    end

    def create_entry_and_job(current_record, type, identifier = nil, index = 0)
      identifier ||= current_record[source_identifier]
      new_entry = find_or_create_entry(send("#{type}_entry_class"),
                                       identifier,
                                       'Bulkrax::Importer',
                                       record_raw_metadata(current_record))
      new_entry.status_info('Pending', importer.current_run)
      if record_deleted?(current_record)
        "Bulkrax::Delete#{type.camelize}Job".constantize.send(perform_method, new_entry, current_run)
      elsif record_remove_and_rerun?(current_record) || remove_and_rerun
        delay = calculate_type_delay(type)
        "Bulkrax::DeleteAndImport#{type.camelize}Job".constantize.set(wait: delay).send(perform_method, new_entry, current_run)
      else
        delay = calculate_type_delay(type, index)
        "Bulkrax::Crc1280Experiment::Import#{type.camelize}Job".constantize.set(wait: delay).send(perform_method, new_entry.id, current_run.id)
      end
    end

    def write_partial_import_file(file)
      import_filename = import_file_path.split('/').last
      partial_import_filename = "#{File.basename(import_filename, '.csv')}_corrected_entries.csv"

      path = File.join(path_for_import, partial_import_filename)
      FileUtils.mv(
        file.path,
        path
      )
      path
    end

    def current_records_for_export
      @current_records_for_export ||= Bulkrax::ParserExportRecordSet.for(
        parser: self,
        export_from: importerexporter.export_from
      )
    end

    def create_new_entries
      # NOTE: The each method enforces the limit, as it can best optimize the underlying queries.
      current_records_for_export.each do |id, entry_class|
        new_entry = find_or_create_entry(entry_class, id, 'Bulkrax::Exporter')
        begin
          entry = ExportWorkJob.perform_now(new_entry.id, current_run.id)
        rescue StandardError => e
          Rails.logger.info("#{e.message} was detected during export")
        end

        self.headers |= entry.parsed_metadata.keys if entry
      end
    end
    alias create_from_collection create_new_entries
    alias create_from_importer create_new_entries
    alias create_from_worktype create_new_entries
    alias create_from_all create_new_entries


    def entry_class
      CrcFolderEntry
    end

    def collection_entry_class
      CsvCollectionEntry
    end

    def file_set_entry_class
      CrcFolderFileSetEntry
    end

    def complex_subject_entry_class
      CrcFolderComplexSubjectEntry
    end

    def complex_session_entry_class
      CrcFolderComplexSessionEntry
    end

    def complex_modality_entry_class
      CrcFolderComplexModalityEntry
    end


    def valid_entry_types
      [collection_entry_class.to_s, file_set_entry_class.to_s, entry_class.to_s]
    end

    # TODO: figure out why using the version of this method that's in the bagit parser
    # breaks specs for the "if importer?" line
    def total
      @total =
        if importer?
          importer.parser_fields['total'] || 0
        elsif exporter?
          limit.to_i.zero? ? current_records_for_export.count : limit.to_i
        else
          0
        end

       @total
    rescue StandardError
      @total = 0
    end

    def records_split_count
      1000
    end

    # export methods

    def write_files
      require 'open-uri'
      folder_count = 0
      # TODO: This is not performant as well; unclear how to address, but lower priority as of
      #       <2023-02-21 Tue>.
      sorted_entries = sort_entries(importerexporter.entries.uniq(&:identifier))
                       .select { |e| valid_entry_types.include?(e.type) }

      group_size = limit.to_i.zero? ? total : limit.to_i
      sorted_entries[0..group_size].in_groups_of(records_split_count, false) do |group|
        folder_count += 1

        CSV.open(setup_export_file(folder_count), "w", headers: export_headers, write_headers: true) do |csv|
          group.each do |entry|
            csv << entry.parsed_metadata
            # TODO: This is precarious when we have descendents of Bulkrax::CsvCollectionEntry
            next if importerexporter.metadata_only? || entry.type == 'Bulkrax::CsvCollectionEntry'

            store_files(entry.identifier, folder_count.to_s)
          end
        end
      end
    end

    def store_files(identifier, folder_count)
      record = Bulkrax.object_factory.find(identifier)
      return unless record

      file_sets = Array.wrap(record) if record.file_set?
      if file_sets.nil? # for valkyrie
        file_sets = record.respond_to?(:file_sets) ? record.file_sets : record.members&.select(&:file_set?)
      end
      file_sets << record.thumbnail if exporter.include_thumbnails && record.thumbnail.present? && record.work?
      file_sets.each do |fs|
        path = File.join(exporter_export_path, folder_count, 'files')
        FileUtils.mkdir_p(path) unless File.exist? path
        file = filename(fs)
        next if file.blank? || fs.original_file.blank?

        io = fs.original_file.respond_to?(:uri) ? open(fs.original_file.uri) : fs.original_file.file.io
        File.open(File.join(path, file), 'wb') do |f|
          f.write(io.read)
          f.close
        end
      end
    rescue Ldp::Gone
      nil
    end

    def export_key_allowed(key)
      new_entry(entry_class, 'Bulkrax::Exporter').field_supported?(key) &&
        key != source_identifier.to_s
    end

    # All possible column names
    def export_headers
      headers = sort_headers(self.headers)

      # we don't want access_control_id exported and we want file at the end
      headers.delete('access_control_id') if headers.include?('access_control_id')

      # add the headers below at the beginning or end to maintain the preexisting export behavior
      headers.prepend('model')
      headers.prepend(source_identifier.to_s)
      headers.prepend('id')

      headers.uniq
    end

    def object_names
      return @object_names if @object_names

      @object_names = mapping.values.map { |value| value['object'] }
      @object_names.uniq!&.delete(nil)

      @object_names
    end

    def sort_entries(entries)
      # always export models in the same order: work, collection, file set
      #
      # TODO: This is a problem in that only these classes are compared.  Instead
      #       We should add a comparison operator to the classes.
      entries.sort_by do |entry|
        case entry.type
        when 'Bulkrax::CsvCollectionEntry'
          '1'
        when 'Bulkrax::CsvFileSetEntry'
          '2'
        else
          '0'
        end
      end
    end

    def sort_headers(headers)
      # converting headers like creator_name_1 to creator_1_name so they get sorted by numerical order
      # while keeping objects grouped together
      headers.sort_by do |item|
        number = item.match(/\d+/)&.[](0) || 0.to_s
        sort_number = number.rjust(4, "0")
        object_prefix = object_names.detect { |o| item.match(/^#{o}/) } || item
        remainder = item.gsub(/^#{object_prefix}_/, '').gsub(/_#{number}/, '')
        "#{object_prefix}_#{sort_number}_#{remainder}"
      end
    end

    # in the parser as it is specific to the format
    def setup_export_file(folder_count)
      path = File.join(importerexporter.exporter_export_path, folder_count.to_s)
      FileUtils.mkdir_p(path) unless File.exist?(path)

      File.join(path, "export_#{importerexporter.export_source}_from_#{importerexporter.export_from}_#{folder_count}.csv")
    end

    # Retrieve file paths for [:file] mapping in records
    #  and check all listed files exist.
    def file_paths
      raise StandardError, 'No records were found' if records.blank?
      return [] if importerexporter.metadata_only?

      @file_paths ||= records.map do |r|
        file_mapping = Bulkrax.field_mappings.dig(self.class.to_s, 'file', :from)&.first&.to_sym || :file
        next if r[file_mapping].blank?

        r[file_mapping].split(/\s*[:;|]\s*/).map do |file_path|
          if File.exist?(file_path) # rubocop:disable Style/GuardClause
            file_path
          else
            raise "File #{file_path} does not exist"
          end
        end
      end.flatten.compact.uniq
    end

    private

    def unique_collection_identifier(collection_hash)
      entry_uid = collection_hash[source_identifier]
      entry_uid ||= if Bulkrax.fill_in_blank_source_identifiers.present?
                      Bulkrax.fill_in_blank_source_identifiers.call(self, records.find_index(collection_hash))
                    else
                      collection_hash[:title].split(/\s*[;|]\s*/).first
                    end

      entry_uid
    end

    # This will create a csv after parsing the crc folders
    # and returns prepared csv file path
    def real_import_file_path
      data = PrepareCsvFromCrcFolder.new(self)
      data.prepare_csv!

      data.csv_file_path
    end

    def calculate_type_delay(type, index = 1)
      if type == 'file_set'
        return 120.seconds + (index * 10.seconds)
      end
      return 1.minute if type == 'work'
      return 0
    end
  end
end
